"""tangent_client."""

# from tangent_client.anomaly_detection.api import AnomalyDetectionAPI
# from tangent_client.forecasting.api import ForecastingAPI
# from tangent_client.insights.insights import Insights
# from tangent_client.julia_core.connector import JuliaConnector
# from tangent_client.utils.license import TangentLicense
# from tangent_client.utils.spark import (
#     SparkParallelProcessingApi as SparkParallelProcessing,
# )

from tangent_client.client import TangentClient

from ._version import (
    __description__,
    __metadata__,
    __url__,
    __version__,
)

__all__ = [
    "TangentClient",
    "__description__",
    "__metadata__",
    "__url__",
    "__version__",
]


import tangent_client.logging as tc_logging

# setup the logging according to our logging.yaml
tc_logging.initialize()


# class TangentWorks:
#     """
#     Main TangentWorks class wrapping all underlying funcionality.

#     The main entry point for interacting with the Tangent Works library, providing access to

#     forecasting, anomaly detection, insights, license and parellel processing in spark.

#     Attributes:
#         - forecasting (ForecastingAPI):
#             Provides methods for forecasting operations such as model building,
#             predictions, root cause analysis (RCA), and automatic forecasting.

#             Available Methods:
#                 - build_model:
#                     Builds a forecasting model using the provided configuration and dataset.
#                 - predict:
#                     Generates predictions using the given configuration, dataset, and forecasting model.
#                 - rca:
#                     Performs root cause analysis (RCA) to identify potential causes
#                     of anomalies or variations.
#                 - auto_forecast:
#                     Automatically builds a forecasting model and generates predictions in a single step.

#         - anomaly_detection (AnomalyDetectionAPI):
#             Provides methods for anomaly detection, including model building,
#             anomaly detection, and root cause analysis (RCA).

#             Available Methods:
#                 - build_model:
#                     Builds an anomaly detection model using the provided configuration and dataset.
#                 - detect:
#                     Detects anomalies in the provided dataset using
#                     the specified anomaly detection model.
#                 - rca:
#                     Performs root cause analysis (RCA) to identify potential causes
#                     of anomalies in the dataset.

#         - insights (Insights):
#             Provides methods for generating insights and interpreting results.

#             Available Methods:
#                 - properties:
#                     Retrieves and processes the variable properties of the given model,
#                     sorted by importance.
#                 - features:
#                     Extracts and processes the features used in the given model.

#         - license (TangentLicense):
#             Provides methods for working with the Tangent License.

#             Available Methods:
#                 - get_token:
#                     Retrieves the Tangent License token from the environment.
#                 - decode_token:
#                     Decodes the Tangent License token.
#                 - validate_json:
#                     Validates the Tangent License token JSON.
#                 - validate_license:
#                     Validates the Tangent License token.

#         - spark_parallel_processing (SparkParallelProcessing):
#             Provides methods for parallel processing in Spark.

#             Available Methods:
#                 - parallelize:
#                     Parallelizes the given dataset in Spark.
#                 - map:
#                     Maps a function to the given dataset in Spark.
#                 - reduce:
#                     Reduces the given dataset

#     Example Usage:
#         ```python
#         from tangent_works import TangentWorks

#         # Initialize TangentWorks
#         tw = TangentWorks()

#         # Use the ForecastingAPI
#         model = tw.forecasting.build_model(configuration=config, dataset=data)
#         predictions = tw.forecasting.predict(configuration=config, dataset=data, model=model)

#         # Use the AnomalyDetectionAPI
#         anomaly_model = tw.anomaly_detection.build_model(configuration=config, dataset=data)
#         anomalies = tw.anomaly_detection.detect(dataset=data, model=anomaly_model)

#         # Use the Insights API
#         insights = tw.insights.properties(model=model)
#         features = tw.insights.features(model=model)
#         ```

#     """

#     def __init__(self) -> None:
#         self._connector = JuliaConnector()
#         self.forecasting = ForecastingAPI(connector=self._connector)
#         self.anomaly_detection = AnomalyDetectionAPI(connector=self._connector)
#         self.insights = Insights()
#         self.license = TangentLicense()
#         self.spark_parallel_processing = SparkParallelProcessing(app_name="TangentWorks")

#     def __repr__(self) -> str:
#         return f"<{self.__class__.__name__}>"

#     def __str__(self) -> str:
#         return self.__repr__()



"""tangent_client.client."""

import io
import json
import os

import pandas as pd
import requests

TANGENT_API_SERVER = os.getenv("TANGENT_API_SERVER", "https://api.tangent.works/")
TANGENT_LICENSE = os.getenv("TANGENT_LICENSE", "")
TANGENT_API_TIMEOUT = os.getenv("TANGENT_API_TIMEOUT", "90")


class TangentClient:
    """Client for the Tangent API."""

    def __init__(self, server: str = "", token: str = "", timeout: str | int = "") -> None:
        self.server = server or TANGENT_API_SERVER
        self.token = token or TANGENT_LICENSE
        self.timeout = int(timeout or TANGENT_API_TIMEOUT)

        self.auto_forecast = AutoForecastClient(self.server, self.token)
        self.forecasting = ForecastingClient(self.server, self.token)
        self.anomaly_detection = AnomalyDetectionClient(server, token)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} server={self.server} timeout={self.timeout}>"

    def __str__(self) -> str:
        return self.__repr__()


class AutoForecastClient:
    def __init__(self, server: str = "", token: str = "", timeout: str | int = "90") -> None:
        self.server = server or TANGENT_API_SERVER
        self.token = token or TANGENT_LICENSE
        self.timeout = int(timeout) or int(TANGENT_API_TIMEOUT)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} server={self.server} timeout={self.timeout}>"

    def __str__(self) -> str:
        return self.__repr__()

    def run(self, configuration, dataset):
        url = self.server + "/auto-forecast"
        headers = {"Authorization": self.token}
        configuration_file = json.dumps(configuration)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            files={
                "dataset": ("dataset", dataset_file, "text/csv"),
                "configuration": (
                    "configuration",
                    configuration_file,
                    "application/json",
                ),
            },
            timeout=self.timeout,
        )
        return resp.json()

    def status(self, id):
        url = self.server + f"/auto-forecast/{id}"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return resp.json()

    def results(self, id):
        url = self.server + f"/auto-forecast/{id}/results"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return pd.read_csv(io.StringIO(resp.text))


class AnomalyDetectionClient:
    """Client for the Tangent AnomalyDetection API."""

    def __init__(self, server: str = "", token: str = "", timeout: str | int = "90") -> None:
        self.server = server or TANGENT_API_SERVER
        self.token = token or TANGENT_LICENSE
        self.timeout = int(timeout) or int(TANGENT_API_TIMEOUT)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} server={self.server} timeout={self.timeout}>"

    def __str__(self) -> str:
        return self.__repr__()

    def build_model(self, configuration, dataset):
        url = self.server + "/anomaly-detection/build-model"
        headers = {"Authorization": self.token}
        configuration_file = json.dumps(configuration)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=self.timeout,
            files={
                "dataset": ("dataset", dataset_file, "text/csv"),
                "configuration": (
                    "configuration",
                    configuration_file,
                    "application/json",
                ),
            },
        )
        return resp.json()

    def detect(self, model, dataset):
        url = self.server + "/anomaly-detection/detect"
        headers = {"Authorization": self.token}
        model_file = json.dumps(model)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=self.timeout,
            files={
                "dataset": ("dataset", dataset_file, "text/csv"),
                "model": ("model", model_file, "application/json"),
            },
        )
        return resp.json()

    def status(self, id):
        url = self.server + f"/anomaly-detection/{id}"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return resp.json()

    def model(self, id):
        url = self.server + f"/anomaly-detection/{id}/model"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return resp.json()

    def results(self, id):
        url = self.server + f"/anomaly-detection/{id}/results"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return pd.read_csv(io.StringIO(resp.text))

    def rca(self, configuration, model, dataset):
        url = self.server + "/anomaly-detection/rca"
        headers = {"Authorization": self.token}
        configuration_file = json.dumps(configuration)
        model_file = json.dumps(model)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=self.timeout,
            files={
                "configuration": (
                    "configuration",
                    configuration_file,
                    "application/json",
                ),
                "model": ("model", model_file, "application/json"),
                "dataset": ("dataset", dataset_file, "text/csv"),
            },
        )
        return resp.json()

    def rca_results(self, id):
        url = self.server + f"/anomaly-detection/{id}/rca-table"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        rca_json = json.loads(resp.text)
        output = []
        for model_index in rca_json.keys():
            rca_json[model_index]
            rca_df = pd.read_csv(io.StringIO(rca_json[model_index]))
            output.append({model_index: rca_df})
        return output


class ForecastingClient:
    """Client for the Tangent Forecasting API."""

    def __init__(self, server: str = "", token: str = "", timeout: str | int = "90") -> None:
        self.server = server
        self.token = token
        self.timeout = int(timeout) or int(TANGENT_API_TIMEOUT)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} server={self.server} timeout={self.timeout}>"

    def __str__(self) -> str:
        return self.__repr__()

    def build_model(self, configuration, dataset):
        url = self.server + "/forecast/build-model"
        headers = {"Authorization": self.token}
        configuration_file = json.dumps(configuration)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=self.timeout,
            files={
                "dataset": ("dataset", dataset_file, "text/csv"),
                "configuration": (
                    "configuration",
                    configuration_file,
                    "application/json",
                ),
            },
        )
        return resp.json()

    def predict(self, configuration, model, dataset):
        url = self.server + "/forecast/predict"
        headers = {"Authorization": self.token}
        configuration_file = json.dumps(configuration)
        model_file = json.dumps(model)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=self.timeout,
            files={
                "configuration": (
                    "configuration",
                    configuration_file,
                    "application/json",
                ),
                "model": ("model", model_file, "application/json"),
                "dataset": ("dataset", dataset_file, "text/csv"),
            },
        )
        return resp.json()

    def status(self, id):
        url = self.server + f"/forecast/{id}"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return resp.json()

    def model(self, id):
        url = self.server + f"/forecast/{id}/model"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return resp.json()

    def results(self, id):
        url = self.server + f"/forecast/{id}/results"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        return pd.read_csv(io.StringIO(resp.text))

    def rca(self, configuration, model, dataset):
        url = self.server + "/forecast/rca"
        headers = {"Authorization": self.token}
        configuration_file = json.dumps(configuration)
        model_file = json.dumps(model)
        dataset_file = dataset.to_csv(index=False)
        resp = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=self.timeout,
            files={
                "configuration": (
                    "configuration",
                    configuration_file,
                    "application/json",
                ),
                "model": ("model", model_file, "application/json"),
                "dataset": ("dataset", dataset_file, "text/csv"),
            },
        )
        return resp.json()

    def rca_results(self, id):
        url = self.server + f"/forecast/{id}/rca-table"
        headers = {"Authorization": self.token}
        resp = requests.request(method="GET", url=url, headers=headers, timeout=self.timeout)
        rca_json = json.loads(resp.text)
        output = []
        for model_index in rca_json.keys():
            rca_json[model_index]
            rca_df = pd.read_csv(io.StringIO(rca_json[model_index]))
            output.append({model_index: rca_df})
        return output



"""
tangent_client._version.

Fetch metadata from the package's pyproject.toml.
The package must be properly installed in order the metadata to be available.

"""

import importlib.metadata

try:
    pkginfo: dict = importlib.metadata.metadata(__package__).json
except importlib.metadata.PackageNotFoundError:  # pragma: no cover
    # fallback if this package is not properly installed
    pkginfo = {}


__metadata__: dict = pkginfo

__author__: str | list[str] = pkginfo.get("author", "unknown")

__author_email__: str | list[str] = pkginfo.get("author_email", "unknown")

__copyright__: str | list[str] = pkginfo.get("license", "unknown")

__description__: str | list[str] = pkginfo.get("summary", "unknown")

__license__: str | list[str] = pkginfo.get("license", "unknown")

__pkg_name__: str | list[str] = pkginfo.get("name", "unknown")

__readme__: str | list[str] = pkginfo.get("description", "unknown")

__title__ = __package__

__url__: str | list[str] = pkginfo.get("project_url", "unknown")

__version__: str | list[str] = pkginfo.get("version", "unknown")



"""tangent_client.logging."""

import contextlib
import importlib
import logging

import libranet_logging

log = logging.getLogger(__name__)
julia_core_log = logging.getLogger("tangent_julia_core")


def initialize() -> None:
    """Configure logging using the logging.yaml file from the package resources."""
    logging_yaml_file = importlib.resources.files("tangent_client").joinpath("logging.yaml")  # Python3.9+
    libranet_logging.initialize(path=logging_yaml_file)
